using System;
using System.IO;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Windows.Forms;
using java.security;
using Org.BouncyCastle.Crypto;

namespace Proyecto
{
    /*
     * Clase FormCrearUsuario: muestra una ventana que permite crear un nuevo usuario
     * Autor: Juan lvaro Muoz Naranjo
     * Fecha: Junio de 2007
     */
    public partial class FormCrearUsuario : Form
    {
        public FormCrearUsuario(ProgressBar barra, System.Windows.Forms.Timer temp)
        {
            InitializeComponent();
            
            barraInvocador = barra;
            temporizador = temp;

        }

        //invocado cuando el usuario pulsa el botn Aceptar
        private void bAceptar_Click(object sender, EventArgs e)
        {
            if (tbNombre.Text == "" || tbEmail.Text == "" ||
                tbFrase.Text == "" || tbConfirmeFrase.Text == "")
            {
                if (temporizador != null)
                {
                    temporizador.Stop();
                    temporizador.Start();
                }
                MessageBox.Show("Debe rellenar todos los campos.");
            }
            //si se ejecuta el programa por primera vez "usuarios.usr" no existe, salta el condicional sin buscar Datos.Usuarios en l
            else if (File.Exists("usuarios.usr") && Datos.Usuarios.buscarUsuarioPorNombre(tbNombre.Text) != null)
            {
                if (temporizador != null)
                {
                    temporizador.Stop();
                    temporizador.Start();
                }
                MessageBox.Show("Ya existe un usuario con ese nombre");
            }
            else if (tbFrase.Text != tbConfirmeFrase.Text)
            {
                if (temporizador != null)
                {
                    temporizador.Stop();
                    temporizador.Start();
                }
                MessageBox.Show("Las frases de acceso introducidas no coinciden");
                tbFrase.Clear();
                tbConfirmeFrase.Clear();
            }
            else if (tbFrase.Text.Length < 6)
            {
                if (temporizador != null)
                {
                    temporizador.Stop();
                    temporizador.Start();
                }
                MessageBox.Show("La frase de acceso debe tener al menos 6 caracteres");
                tbFrase.Clear();
                tbConfirmeFrase.Clear();
            }
            else
            {
                try
                {
                    if (temporizador != null) temporizador.Stop();
                    if (barraInvocador != null) barraInvocador.Increment(10);
                    //generamos las claves pblicas del nuevo usuario
                    PublicKey clavePublicaRSA1024 = Utilidades.generarParDeClavesRSAconPBE(1024, tbFrase.Text.ToCharArray()).getPublic();
                    if (barraInvocador != null) barraInvocador.Increment(10);
                    AsymmetricKeyParameter clavePublicaElGamal512 = Utilidades.generarParDeClavesElGamalconPBE(512, tbFrase.Text.ToCharArray()).Public;
                    if (File.Exists("listaClaveSistema.lcs") && File.Exists("usuarios.usr"))
                    {
                        //aadimos el usuario
                        Datos.Usuarios.aadirUsuario(tbNombre.Text, tbEmail.Text, true,
                            new ParClavePublicaNombreAlgoritmo("RSA", clavePublicaRSA1024.getEncoded()),
                            new ParClavePublicaNombreAlgoritmo("ElGamal", Utilidades.codificarClaveElGamal(clavePublicaElGamal512)));
                        if (barraInvocador != null) barraInvocador.Increment(10);
                        //guardamos el nuevo archivo de usuarios
                        Datos.Usuarios.guardarEnArchivoCifrado("usuarios.usr.temp", "DES/ECB/PKCS5Padding", Datos.ClaveSistema);
                        if (barraInvocador != null) barraInvocador.Increment(20);
                        //generamos el nuevo archivo del sistema                    
                        ListaClaveSistema listaClaveSistema = new ListaClaveSistema("listaClaveSistema.lcs");
                        if (barraInvocador != null) barraInvocador.Increment(10);
                        listaClaveSistema.aadirEstructuraClaveSistema(tbNombre.Text, Datos.ClaveSistema, "RSA", clavePublicaRSA1024);
                        if (barraInvocador != null) barraInvocador.Increment(10);
                        listaClaveSistema.guardarListaEnArchivo("listaClaveSistema.lcs.temp");
                        if (barraInvocador != null) barraInvocador.Increment(20);
                        File.Replace("usuarios.usr.temp", "usuarios.usr", "usuarios.usr.old");
                        File.Replace("listaClaveSistema.lcs.temp", "listaClaveSistema.lcs", "listaClaveSistema.lcs.old");
                        if (barraInvocador != null) barraInvocador.Increment(10);
                    }
                    //cuando se crea un usuario por primera vez
                    else
                    {
                        //generamos el archivo de usuarios
                        Usuarios usuarios = new Usuarios();
                        usuarios.aadirUsuario(tbNombre.Text, tbEmail.Text, true,
                             new ParClavePublicaNombreAlgoritmo("RSA", clavePublicaRSA1024.getEncoded()),
                             new ParClavePublicaNombreAlgoritmo("ElGamal", Utilidades.codificarClaveElGamal(clavePublicaElGamal512)));
                        usuarios.guardarEnArchivoCifrado("usuarios.usr", "DES/ECB/PKCS5Padding", Datos.ClaveSistema);

                        //generamos el archivo del sistema
                        ListaClaveSistema listaClaveSistema = new ListaClaveSistema();
                        listaClaveSistema.aadirEstructuraClaveSistema(tbNombre.Text,Datos.ClaveSistema,"RSA",clavePublicaRSA1024);
                        listaClaveSistema.guardarListaEnArchivo("listaClaveSistema.lcs");

                    }
                    if (temporizador != null) temporizador.Start();
                    this.Close();                    
                }
                catch (IOException ioe)
                {
                    if (temporizador != null) temporizador.Start();
                    MessageBox.Show(ioe.Source + "\n" + ioe.Message + "\n" + ioe.ToString());
                }
                if (barraInvocador != null) barraInvocador.Value = 0;
                if (temporizador != null) temporizador.Start();
            }
        }

        //invocado cuando el usuario pulsa el botn Cancelar
        private void bCancelar_Click(object sender, EventArgs e)
        {
            if (temporizador != null)
            {
                temporizador.Stop();
                temporizador.Start();
            }
            this.Dispose();
            this.Close();
        }
    }
}