using System;
using System.IO;
using System.Collections;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using javax.crypto;
using java.security;
using java.security.spec;
using Org.BouncyCastle.Crypto;

namespace Proyecto
{
    /*
     * Clase FormPedirContrasea: lleva el peso de la ejecucin
     * Se ocupa de validar el usuario que inicia la aplicacin, situar un icono
     * en la bandeja del sistema y mantener a la escucha dos objetos FileSystemWatcher
     * Autor: Juan lvaro Muoz Naranjo
     * Fecha: Junio de 2007
     */
    public partial class FormPedirContrasea : Form
    {
        private const string nombreCarpeta = "C:\\CarpetaCifrada";
        private const string extension = ".cifrado";

        //private bool parametrosObtenidos = false;
        private string informadoNoPermisos = ""; //indica si ya se ha informado al usuario de que no tiene permiso para descifrar el archivo
        private string informadoModificado = ""; //indica si ya se ha infomrado al usuario de que el archivo ha sido modificado externamente
        private string nombreArchivo;
        private string algoritmoSimetrico;
        private int longClaveSimetrica;
        private string algoritmoAsimetrico;
        private ArrayList usuariosPermisos;

        internal NotifyIcon IconoBandeja
        {
            get
            {
                return notifyIcon;
            }
        }

        public FormPedirContrasea()
        {
            InitializeComponent();
        }

        //invocado cuando el usuario pulsa el botn Salir
        //cierra la ventana y el programa
        private void salir_Click(object sender, EventArgs e)
        {
            this.Close();
        }

        //invocado cuando el usuario pulsa el botn Aceptar
        private void aceptar_Click(object sender, EventArgs e)
        {
            temporizador.Stop();
            if (tbUsuario.Text.Length != 0 && tbFrase.Text.Length != 0)
            {
                try
                {
                    barraProgreso.Increment(10);
                    Datos.ClavePrivadaRSAUsuario = Utilidades.generarParDeClavesConPBE(tbFrase.Text.ToCharArray(), "RSA", 1024).getPrivate();
                    Datos.ClavePrivadaElGamalUsuario = Utilidades.generarParDeClavesElGamalconPBE(512, tbFrase.Text.ToCharArray()).Private;
                    barraProgreso.Increment(30);
                    ListaClaveSistema listaClaveSistema = new ListaClaveSistema("listaClaveSistema.lcs");
                    barraProgreso.Increment(20);
                    Datos.ClaveSistema = listaClaveSistema.buscarEstructuraClaveSistema(tbUsuario.Text, "RSA", Datos.ClavePrivadaRSAUsuario);
                    
                    //si se encontr el usuario, Datos.ClaveSistema!=null
                    //EL USUARIO NO ES VLIDO
                    if (Datos.ClaveSistema == null)
                    {
                        barraProgreso.Increment(20);
                        MessageBox.Show("Usuario o frase de acceso incorrectos", "", MessageBoxButtons.OK);
                        //limpiamos Datos
                        Datos.limpiar();
                        //reseteamos el formulario para que el usuario vuelva a introducir sus datos
                        barraProgreso.Value = 0;
                        tbUsuario.Clear();
                        tbFrase.Clear();
                        temporizador.Start();
                        return;
                    }
                    //EL USUARIO ES VLIDO
                    else
                    {
                        Datos.NombreUsuario = tbUsuario.Text;
                        Datos.Usuarios = new Usuarios("usuarios.usr", "DES/ECB/PKCS5Padding", Datos.ClaveSistema);
                        barraProgreso.Increment(20);

                        notifyIcon.Visible = true;
                        //llamada a escucharEnCarpeta()
                        escucharEnCarpeta();
                        temporizador.Start();
                        
                        //habilitamos todas las opciones del icono de la bandeja
                        //(al cambiar de usuario se deshabilitan)
                        foreach (object o in this.contextMenuStrip.Items)
                            ((ToolStripMenuItem)o).Enabled = true;
                        //ocultamos el formulario
                        this.Hide();
                    }

                }
                catch (FileNotFoundException fnfe)
                {
                    temporizador.Start();
                    MessageBox.Show(fnfe.Message+"\nEs posible que se haya comprometido la seguridad del sistema.", "", MessageBoxButtons.OK);
                    this.Close();
                    this.Dispose();
                }
                catch (IOException ioe)
                {
                    temporizador.Start();
                    MessageBox.Show(ioe.Message, "", MessageBoxButtons.OK);
                    this.Close();
                    this.Dispose();
                }
                catch (Exception exc)
                {
                    temporizador.Start();
                    MessageBox.Show(exc.Message, "", MessageBoxButtons.OK);
                    this.Close();
                    this.Dispose();
                }
            }
            else if (tbUsuario.Text.Length == 0 && tbFrase.Text.Length == 0)
            {
                temporizador.Start();
                MessageBox.Show("Debe introducir un nombre de usuario y una frase de acceso", "", MessageBoxButtons.OK);
            }
            else if (tbUsuario.Text.Length == 0)
            {
                temporizador.Start();
                MessageBox.Show("Debe introducir un nombre de usuario", "", MessageBoxButtons.OK);
            }
            else if (tbFrase.Text.Length == 0)
            {
                temporizador.Start();
                MessageBox.Show("Debe introducir una frase de acceso", "", MessageBoxButtons.OK);
            }
            temporizador.Start();
        }

        //invocado cuando el usuario pulsa la opcin Salir del icono de la bandeja
        //del sistema
        private void salirToolStripMenuItem_Click(object sender, EventArgs e)
        {
            //eliminamos las referencias "delicadas"
            Datos.limpiar();

            this.Close();
        }

        //pone a la estucha los objetos FilyeSystemWatcher
        private void escucharEnCarpeta()
        {
            watcherEntrada.EnableRaisingEvents = true;
            watcherSalida.EnableRaisingEvents = true;

            notifyIcon.ShowBalloonTip(500,"","C:\\CarpetaCifrada activada",ToolTipIcon.None);
         }

        //manejador de enventos de watcherEntrada
        private void OnChangedEntrada(object source, FileSystemEventArgs e)
        {          
            try
            {
                //compruebo que no tenga extensin ".cifrado", porque si no, al crearse un nuevo ".cifrado",
                //cifrara este de nuevo, volviendo a cifrar el resultado, y as sucesivamente
                if (!e.FullPath.EndsWith(extension))// & e.ChangeType.Equals(WatcherChangeTypes.Changed))
                {
                    temporizador.Stop();
                    if (e.ChangeType.Equals(WatcherChangeTypes.Created))
                    {
                        FormPedirParametrosCifrado pedirParametros = new FormPedirParametrosCifrado(e.Name, temporizador);
                        pedirParametros.ShowDialog();

                        if (pedirParametros.Resultado == DialogResult.Cancel)
                        {
                            MessageBox.Show("Operacin cancelada. El archivo no se cifrar.");
                            return;
                        }

                        nombreArchivo = e.FullPath;
                        algoritmoSimetrico = pedirParametros.AlgoritmoSimetrico;
                        longClaveSimetrica = pedirParametros.LongClaveSimetrica;
                        algoritmoAsimetrico = pedirParametros.AlgoritmoAsimetrico;
                        usuariosPermisos = pedirParametros.NombresUsuariosPermiso;
                        //parametrosObtenidos = true;
                    }

                    //creamos la clave simtrica que cifrar la informacin del archivo
                    SecretKey claveSimetrica = null;
                    if (algoritmoSimetrico == "DES" || algoritmoSimetrico == "DESede")
                        claveSimetrica = Utilidades.generarClaveDES();
                    else if (algoritmoSimetrico == "AES")
                    {
                        if (longClaveSimetrica == 128 ||
                            longClaveSimetrica == 192 ||
                            longClaveSimetrica == 256)
                            claveSimetrica = Utilidades.generarClaveAES(longClaveSimetrica);
                        else
                        {
                            temporizador.Start();
                            MessageBox.Show("Error en la longitud de clave");
                            return;
                        }
                    }
                    else if (algoritmoSimetrico == "Blowfish")
                    {
                        if (longClaveSimetrica == 128 ||
                            longClaveSimetrica == 320)
                            claveSimetrica = Utilidades.generarClaveBlowfish(longClaveSimetrica);
                        else
                        {
                            temporizador.Start();
                            MessageBox.Show("Error en la longitud de clave");
                            return;
                        }
                    }
                    else
                    {
                        temporizador.Start();
                        return;
                    }

                    //creamos las cabeceras del archivo
                    Cabeceras cabeceras = new Cabeceras();
                    if (algoritmoAsimetrico == "RSA")
                    {
                        KeyFactory factory = KeyFactory.getInstance("RSA");
                        PublicKey clavePublica;
                        foreach (Object nombre in usuariosPermisos)
                        {
                            clavePublica = factory.generatePublic(new X509EncodedKeySpec(Datos.Usuarios.buscarClavePublicaPorNombre((string)nombre, "RSA")));
                            cabeceras.aadirCabecera((string)nombre, algoritmoSimetrico, "", claveSimetrica.getEncoded(), clavePublica);
                        }
                    }
                    else if (algoritmoAsimetrico == "ElGamal")
                    {
                        AsymmetricKeyParameter clavePublicaRSA;
                        foreach (Object nombre in usuariosPermisos)
                        {
                            clavePublicaRSA = Utilidades.recuperarClaveElGamal(Datos.Usuarios.buscarClavePublicaPorNombre((string)nombre, "ElGamal"));
                            cabeceras.aadirCabecera((string)nombre, algoritmoSimetrico, "", claveSimetrica.getEncoded(), clavePublicaRSA);
                        }
                    }
                    //slo por si acaso
                    if (cabeceras.Tamao == 0)
                    {
                        temporizador.Start();
                        MessageBox.Show("0 cabeceras para aadir");
                        return;
                    }

                    CifrarArchivoYCabeceras.cifrarConCabeceras(e.FullPath, e.FullPath + extension, 
                        cabeceras, algoritmoSimetrico, 
                        Utilidades.generarVectorInicializacion(Utilidades.tamaoBloque(algoritmoSimetrico)),
                        claveSimetrica,Datos.NombreUsuario,Datos.ClavePrivadaRSAUsuario);
                    //borramos el archivo original creado en la carpeta, para que slo quede el cifrado
                    //debera controlar las excepciones
                    File.Delete(e.FullPath);

                    //parametrosObtenidos = false;
                    nombreArchivo = null;
                    algoritmoSimetrico = null;
                    algoritmoAsimetrico = null;
                    usuariosPermisos = null;
                    temporizador.Start();
                }
                return;
            }
            catch (FileNotFoundException /*fnfe*/)
            {
                temporizador.Start();
                return;
            }
            catch (System.IO.IOException /*ioe*/)
            {
                temporizador.Start();
                return;
            }

            catch (Exception exc)
            {
                temporizador.Start();
                MessageBox.Show("OnChangedEntrada() " + exc.GetType() + ": " + exc.Message);
                return;
            }
        }

        //manejador para el objeto watcherSalida
        private void OnChangedSalida(object source, FileSystemEventArgs e)
        {
            string firmante;

            try
            {
                //compruebo que el evento no se haya producido en la carpeta cifrada, 
                //para evitar que descifre el ".cifrado" cada vez que se crea
                if (!e.FullPath.StartsWith(nombreCarpeta) & e.ChangeType.Equals(WatcherChangeTypes.Changed))
                {
                    temporizador.Stop();
                    //KeyFactory factory = KeyFactory.getInstance("RSA");
                    //PublicKey clavePublicaRSA = factory.generatePublic(new X509EncodedKeySpec(Datos.Usuarios.buscarClavePublicaPorNombre(Datos.NombreUsuario, "RSA")));
                    if (CifrarArchivoYCabeceras.descifrarConCabeceras(e.FullPath, e.FullPath.Substring(0, e.FullPath.Length - extension.Length),
                        Datos.ClavePrivadaRSAUsuario, Datos.ClavePrivadaElGamalUsuario, Datos.Usuarios, out firmante) == false)
                    {
                        if (informadoModificado.CompareTo(e.FullPath) != 0)
                        {
                            informadoModificado = e.FullPath;
                            temporizador.Start();
                            MessageBox.Show("El archivo ha sido modificado externamente.");
                            informadoModificado = "";
                            return;
                        }
                        temporizador.Start();
                        return;
                    }
                    //borramos el archivo original creado en la carpeta, para que slo quede el cifrado
                    //debera controlar las excepciones
                    File.Delete(e.FullPath);
                    temporizador.Start();
                    MessageBox.Show(Path.GetFileName(e.FullPath) + "\nFirmante: " + firmante);
                }
            }
            catch (FileNotFoundException)
            {
                temporizador.Start();
                return;
            }
            catch (System.IO.IOException)
            {
                temporizador.Start();
                return;
            }
            //DEVUELVE EL ARCHIVO A LA CARPETA, (P.EJ. C:\CarpetaCifrada)
            //SI EL ARCHIVO FUE COPIADO, NO MOVIDO, SE LIMITA A ELIMINAR EL NUEVO ARCHIVO
            catch (ExcepcionCabeceraNoEncontrada ecne)
            {
                if (informadoNoPermisos.CompareTo(e.FullPath) != 0)
                {
                    informadoNoPermisos = e.FullPath;
                    temporizador.Start();
                    MessageBox.Show(ecne.Message);
                    informadoNoPermisos = "";
                }

                //si no se encontr la cabecera el usuario no tiene acceso, devolvemos el
                //archivo a la carpeta
                //si el archivo origen existe, slo se copi fuera, borramos el nuevo y listo
                if (File.Exists(nombreCarpeta + "\\" + Path.GetFileName(e.Name)))
                {
                    try
                    {
                        File.Delete(e.FullPath);
                    }
                    catch (Exception)
                    {
                        temporizador.Start();
                        return;
                    }
                }
                else
                {
                    try
                    {
                        File.Move(e.FullPath, nombreCarpeta + "\\" + Path.GetFileName(e.Name));
                    }
                    catch (Exception excepcionMover)
                    {
                        temporizador.Start();
                        MessageBox.Show(excepcionMover.Message);
                        return;
                    }
                }
                temporizador.Start();
                return;
            }
            catch (ExcepcionFirmaNoVerificada efnv)
            {
                temporizador.Start();
                MessageBox.Show(efnv.Message + "\nFirmante: " + efnv.Firmante);
                return;
            }
            catch (Exception exc)
            {
                temporizador.Start();
                MessageBox.Show("EO OnChangedSalida(): " + exc.Message);
                return;
            }
        }

        //invocado cuando el usuario pulsa la opcin Usuarios del icono de la bandeja
        //del sistema
        //muestra la ventana de gestin de usuarios
        private void usuariosToolStripMenuItem_Click(object sender, EventArgs e)
        {
            temporizador.Stop();
            temporizador.Start();
            FormUsuarios formUsuarios = new FormUsuarios(temporizador);
            formUsuarios.Show();
        }

        //invocado cuando el usuario pulsa la opcin Acerca De... del icono de la bandeja
        //del sistema
        //muestra la ventana de informacin
        private void acercaDeToolStripMenuItem_Click(object sender, EventArgs e)
        {
            temporizador.Stop();
            temporizador.Start();
            (new FormAcercaDe(temporizador)).ShowDialog();
        }

        private void IntervaloExcedido(object sender, EventArgs e)
        {
            this.notifyIcon.ShowBalloonTip(2000, "", "Cerrando la aplicacin...", ToolTipIcon.None);
            System.Threading.Thread.Sleep(2000);
            this.Close();
        }

        private void cambiarDeUsuarioToolStripMenuItem_Click(object sender, EventArgs e)
        {
            temporizador.Stop();
            temporizador.Start();

            //desactivamos las opciones del icono de la bandeja
            //para que el usuario no pueda hacer nada raro mientras se cambia de usuario
            ToolStripMenuItem item;
            foreach (object o in this.contextMenuStrip.Items)
            {
                item = (ToolStripMenuItem)o;
                if (item.Text != "Salir")
                    item.Enabled = false;
            }

            //borramos los datos del usuairo anterior
            Datos.limpiar();
            //preparamos el formulario y lo mostramos
            this.tbUsuario.Clear();
            this.tbFrase.Clear();
            this.barraProgreso.Value = 0;
            this.tbUsuario.TabIndex = 0;
            this.tbFrase.TabIndex = 1;
            this.Show();
        }
    }
}