using System;
using System.Text;
using System.IO;
using System.Runtime.Serialization.Formatters.Binary;
using java.security;
using javax.crypto;
using javax.crypto.spec;
using Org.BouncyCastle.Crypto;
using Org.BouncyCastle.Crypto.Generators;
using Org.BouncyCastle.Crypto.Parameters;
using Org.BouncyCastle.Security;

namespace Proyecto
{
    /*
     * Clase Utilidades: contiene una miscelnea de mtodos, principalmente de conversin
     * y de generacin de claves (aleatorias o PBE)
     * Autor: Juan lvaro Muoz Naranjo
     * Fecha: Junio de 2007
     */
    class Utilidades
    {
        //convierte char[] a byte[]
        internal static byte[] charsABytes(char[] vectorChars)
        {
            byte[] vectorBytes = null;

            if (vectorChars != null)
            {
                vectorBytes = new byte[vectorChars.Length];
                for (int i = 0; i < vectorChars.Length; i++)
                    vectorBytes[i] = (byte)vectorChars[i];
            }
            return vectorBytes;
        }

        //convierte byte[] a char[]
        internal static char[] bytesAChars(byte[] vectorBytes)
        {
            char[] vectorChars = null;

            if (vectorBytes != null)
            {
                vectorChars = new char[vectorBytes.Length];
                for (int i = 0; i < vectorBytes.Length; i++)
                    vectorChars[i] = (char)vectorBytes[i];
            }
            return vectorChars;
        }

        //convierte char[] a String
        internal static string charsAString(char[] vectorChars)
        {
            StringBuilder sb = new StringBuilder();

            sb.Append(vectorChars);
            
            return sb.ToString();
        }

        //convierte byte[] a String
        internal static string bytesAString(byte[] vectorBytes)
        {
            return charsAString(bytesAChars(vectorBytes));
        }

        //compara dos vectores de bytes y devuelve true si son iguales
        public static Boolean compararVectores(byte[] v1, byte[] v2)
        {
            if (v1.Length != v2.Length)
                return false;
            for (int i = 0; i < v1.Length; i++)
            {
                if (v1[i] != v2[i])
                    return false;
            }
            return true;
        }

        //mantiene la consola abierta al terminar la ejecucin
        internal static void terminar()
        {
            Console.WriteLine("\nPulsa intro para terminar...");
            Console.ReadLine();
        }


        ////////////////////////////////////////////
        //MTODOS RELACIONADOS CON LA CRIPTOGRAFA
        ////////////////////////////////////////////

        //genera un vector de inicializacin del tamao especificado (en bytes) en funcin de una semilla
        internal static byte[] generarVectorInicializacion(int tamao, long semilla)
        {
            byte[] vi = new byte[tamao];

            java.security.SecureRandom random = java.security.SecureRandom.getInstance("SHA1PRNG");
            random.setSeed(semilla);
            random.nextBytes(vi);

            return vi;
        }

        //genera un vector de inicializacin del tamao especificado (en bytes)
        internal static byte[] generarVectorInicializacion(int tamao)
        {
            byte[] vi = new byte[tamao];

            java.security.SecureRandom random = java.security.SecureRandom.getInstance("SHA1PRNG");
            random.nextBytes(vi);

            return vi;
        }


        //genera una clave DES en funcin de una semilla
        internal static SecretKey generarClaveDES(long semilla)
        {
            java.security.SecureRandom random = java.security.SecureRandom.getInstance("SHA1PRNG");
            random.setSeed(semilla);

            KeyGenerator keyGen = KeyGenerator.getInstance("DES");
            keyGen.init(random);
            return keyGen.generateKey();
        }

        //genera una clave DES aleatoria
        internal static SecretKey generarClaveDES()
        {
            java.security.SecureRandom random = java.security.SecureRandom.getInstance("SHA1PRNG");

            KeyGenerator keyGen = KeyGenerator.getInstance("DES");
            keyGen.init(random);
            return keyGen.generateKey();
        }

        //genera una clave DES a partir de una contrasea
        internal static SecretKeySpec generarClaveDESConPBE(char[] contrasea)
        {
            MessageDigest md;

            int iteraciones = 5;
            byte[] contraseaBytes;
            byte[] buffer;

            md = MessageDigest.getInstance("SHA-1");
            contraseaBytes = Utilidades.charsABytes(contrasea);
            //primera iteracin
            buffer = md.digest(contraseaBytes);
            //resto de iteraciones
            for (int i = 0; i < iteraciones - 1; i++)
                buffer = md.digest(buffer);
            //creamos la contrasea
            return new SecretKeySpec(buffer, "DES");
        }

        //genera una clave AES del tamao especificado y en funcin de una semilla
        internal static SecretKey generarClaveAES(int tamao, long semilla)
        {
            java.security.SecureRandom random = java.security.SecureRandom.getInstance("SHA1PRNG");
            random.setSeed(semilla);

            KeyGenerator keyGen = KeyGenerator.getInstance("AES");
            keyGen.init(tamao, random);
            return keyGen.generateKey();
        }

        //genera una clave AES aleatoria del tamao especificado
        internal static SecretKey generarClaveAES(int tamao)
        {
            java.security.SecureRandom random = java.security.SecureRandom.getInstance("SHA1PRNG");

            KeyGenerator keyGen = KeyGenerator.getInstance("AES");
            keyGen.init(tamao, random);
            return keyGen.generateKey();
        }

        //genera una clave Blowfish del tamao especificado y en funcin de una semilla
        //SunJCE acepta cualquier tamao de clave entre 64 y 448 bits
        //(aunque Blowfish usa tamaos entre 32 y 448)
        internal static SecretKey generarClaveBlowfish(int tamao, long semilla)
        {
            java.security.SecureRandom random = java.security.SecureRandom.getInstance("SHA1PRNG");
            random.setSeed(semilla);

            KeyGenerator keyGen = KeyGenerator.getInstance("Blowfish");
            keyGen.init(tamao, random);
            return keyGen.generateKey();
        }

        //genera una clave Blowfish aleatoria del tamao especificado
        //SunJCE acepta tamaos de clave entre 64 y 448 bits
        //(aunque Blowfish usa tamaos entre 32 y 448)
        internal static SecretKey generarClaveBlowfish(int tamao)
        {
            java.security.SecureRandom random = java.security.SecureRandom.getInstance("SHA1PRNG");

            KeyGenerator keyGen = KeyGenerator.getInstance("Blowfish");
            keyGen.init(tamao, random);
            return keyGen.generateKey();
        }

        //genera un par de claves del algoritmo especificado en funcin de una contrasea. Se usa para claves RSA.
        //Para generar el par de claves hacemos un resumen de la contrasea "iteraciones" 
        //veces. El byte[] resultado lo usamos de semilla para un generador de nmeros aleatorios,
        //el cual usamos para generar las claves.
        internal static KeyPair generarParDeClavesConPBE(char[] contrasea, string algoritmo, int longitudClave)
        {
            MessageDigest md;

            int iteraciones = 5;
            byte[] contraseaBytes;
            byte[] buffer;

            md = MessageDigest.getInstance("SHA-1");
            contraseaBytes = Utilidades.charsABytes(contrasea);
            //primera iteracin
            buffer = md.digest(contraseaBytes);
            //resto de iteraciones
            for (int i = 0; i < iteraciones - 1; i++)
                buffer = md.digest(buffer);
            //creamos la contrasea
            KeyPairGenerator kpGen = KeyPairGenerator.getInstance(algoritmo);
            //inicializamos el generador de nmeros aleatorios con el byte[]
            //obtenido de resumir la contrasea
            java.security.SecureRandom random = java.security.SecureRandom.getInstance("SHA1PRNG");
            random.setSeed(buffer);
            kpGen.initialize(longitudClave, random);
            return kpGen.generateKeyPair();
        }

        //genera un par de claves RSA en funcin de una contrasea
        //Para generar el par de claves hacemos un resumen de la contrasea "iteraciones" 
        //veces. El byte[] resultado lo usamos de semilla para un generador de nmeros aleatorios,
        //el cual usamos para generar las claves.
        internal static KeyPair generarParDeClavesRSAconPBE(int tamao, char[] contrasea)
        {
            const int vueltas = 5;
            byte[] resumen = charsABytes(contrasea);

            //generamos un resumen "vueltas" veces
            MessageDigest md = MessageDigest.getInstance("SHA-1");
            resumen = md.digest(resumen);
            for (int i = 1; i < vueltas; i++)
                resumen = md.digest(resumen);
            //generamos el par de claves
            java.security.SecureRandom random = java.security.SecureRandom.getInstance("SHA1PRNG");
            random.setSeed(resumen);
            KeyPairGenerator kpGen = KeyPairGenerator.getInstance("RSA");
            kpGen.initialize(tamao, random);
            return kpGen.generateKeyPair();
        }

        //genera un par de claves El Gamal aleatorio del tamao especificado
        //Usa la librera Bouncy Castle
        internal static AsymmetricCipherKeyPair generarParDeClavesElGamal(int tamao)
        {
            Org.BouncyCastle.Security.SecureRandom random = new Org.BouncyCastle.Security.SecureRandom();

            ElGamalParametersGenerator elGamalParamsGenerator = new ElGamalParametersGenerator();
            elGamalParamsGenerator.Init(tamao, 90, random);
            ElGamalParameters elGamalParams = elGamalParamsGenerator.GenerateParameters();
            ElGamalKeyGenerationParameters elGamalKeyGenerationParams = new ElGamalKeyGenerationParameters(random, elGamalParams);

            ElGamalKeyPairGenerator kpGen = new ElGamalKeyPairGenerator();
            //necesitamos un objeto ElGamalKeyGenerationParameters
            kpGen.Init(elGamalKeyGenerationParams);

            return kpGen.GenerateKeyPair();
        }

        //genera un par de claves El Gamal del tamao especificado basndose en una contrasea
        //Usa la librera Bouncy Castle
        internal static AsymmetricCipherKeyPair generarParDeClavesElGamalconPBE(int tamao, char[] contrasea)
        {
            const int vueltas = 5;
            byte[] resumen = charsABytes(contrasea);

            //generamos un resumen "vueltas" veces
            MessageDigest md = MessageDigest.getInstance("SHA-1");
            resumen = md.digest(resumen);
            for (int i = 1; i < vueltas; i++)
                resumen = md.digest(resumen);
            //inicializamos un objeto SecureRandom con el resumen
            Org.BouncyCastle.Security.SecureRandom random = new Org.BouncyCastle.Security.SecureRandom(resumen);

            ElGamalParametersGenerator elGamalParamsGenerator = new ElGamalParametersGenerator();
            elGamalParamsGenerator.Init(tamao, 90, random);
            ElGamalParameters elGamalParams = elGamalParamsGenerator.GenerateParameters();
            ElGamalKeyGenerationParameters elGamalKeyGenerationParams = new ElGamalKeyGenerationParameters(random, elGamalParams);

            ElGamalKeyPairGenerator kpGen = new ElGamalKeyPairGenerator();
            //necesitamos un objeto ElGamalKeyGenerationParameters
            kpGen.Init(elGamalKeyGenerationParams);

            return kpGen.GenerateKeyPair();
        }

        //devuelve una clave ElGamal en bytes
        internal static byte[] codificarClaveElGamal(AsymmetricKeyParameter claveElGamal)
        {
            MemoryStream memStream = new MemoryStream();
            BinaryFormatter formatter = new BinaryFormatter();

            formatter.Serialize(memStream, claveElGamal);
            memStream.Close();

            return memStream.ToArray();
        }

        //devuelve una clave ElGamal en forma de AsymmetricKeyParameter a partir de
        //su serializacin en bytes
        internal static AsymmetricKeyParameter recuperarClaveElGamal(byte[] claveEnBytes)
        {
            MemoryStream memStream = new MemoryStream(claveEnBytes);
            BinaryFormatter formatter = new BinaryFormatter();

            return (AsymmetricKeyParameter)formatter.Deserialize(memStream);
        }

        //devuelve el tamao de bloque del algoritmo indicado
        internal static int tamaoBloque(string algoritmo)
        {
            if (algoritmo == "DES" || algoritmo == "DESede")
                return 8;   //64 bits = 8 bytes
            if (algoritmo == "AES")
                return 16;  //128 bits = 16 bytes
            if (algoritmo == "Blowfish")
                return 8;   //64 bits = 8 bytes
            else 
                return 0;
        }
    }
}
